/*
   FoRTh Modoki
   fgraph.c ver 1.3.1
   Copyright (C) 1999 Tomohide Naniwa. All rights reserved.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   Last modified: May 15, 1999.
*/

#include <X11/Xlib.h>
#include "frtm.h"

#define MAXCOL 16

static int gflag = 0;
static Display *d;
static Window w0;
static int palette[][MAXCOL] = {
  {0, 0,   0,  0, 42, 42, 42, 42, 21, 21, 21, 21, 63, 63, 63, 63},
  {0, 0,  42, 42,  0,  0, 42, 42, 21, 21, 63, 63, 21, 21, 63, 63},
  {0, 42, 0,  42,  0,  42, 0, 42, 21, 63, 21, 63, 21, 63, 21, 63}
};
static struct {
  Font f;
  int asc, lbe;
} font_dat[2];
static GC gc[MAXCOL];
static long pxl[MAXCOL];

static int cx, cy, color, bgcolor, font;

void f_gm()
{
  unsigned long black, white;
  XEvent e;
  Colormap cmap;
  XColor c1;
  Font f;
  XFontStruct *fs;
  int i;
  int asc, lbe;

  color = 15;
  bgcolor = 0;
  font = 0;
  if (gflag) return;
  if ((d = XOpenDisplay(NULL)) == NULL) return;
  gflag = 1;
  black = BlackPixel(d, 0);
  white = WhitePixel(d, 0);
  f = XLoadFont(d, "a14");
  fs = XQueryFont(d, f);
  asc = fs->max_bounds.ascent;
  lbe = fs->max_bounds.lbearing;
  font_dat[0].f = f;
  font_dat[0].asc = asc;
  font_dat[0].lbe = lbe;
  f = XLoadFont(d, "7x14bold");
  fs = XQueryFont(d, f);
  asc = fs->max_bounds.ascent;
  lbe = fs->max_bounds.lbearing;
  font_dat[1].f = f;
  font_dat[1].asc = asc;
  font_dat[1].lbe = lbe;
  w0 = XCreateSimpleWindow(d, RootWindow(d, 0),
			   100, 100, 640, 480, 1, white, black);
/* initialize GC for paint */
  for(i = 0; i < MAXCOL; ++i) {
    gc[i] = XCreateGC(d, w0, 0, 0);
    XSetForeground(d, gc[i], white);
    XSetBackground(d, gc[i], black);
  }
  cmap = DefaultColormap(d, 0);
  for(i = 0; i < MAXCOL; ++i) {
    c1.red = (palette[0][i]+1L)*1024-1;
    c1.green = (palette[1][i]+1L)*1024-1;
    c1.blue = (palette[2][i]+1L)*1024-1;
    XAllocColor(d, cmap, &c1);
    XSetFillStyle(d, gc[i], FillOpaqueStippled);
    XSetForeground(d, gc[i], c1.pixel);
    pxl[i] = c1.pixel;
  }

  XSelectInput(d, w0, ExposureMask);
  XMapWindow(d, w0);
  XFlush(d);

  while(1) {
    XNextEvent(d, &e);
    if (e.type == Expose && e.xexpose.count == 0) break;
  }
  XSelectInput(d, w0, NoEventMask);
  XFlush(d);
}

void f_tm()
{
  if (! gflag) return;
  XCloseDisplay(d);
  gflag = 0;
}

void f_cls()
{
  if (! gflag) return;
  XClearWindow(d, w0);
  XFlush(d);
}

void f_moveto()
{
  int x, y;
  y = pop(); EC;
  x = pop(); EC;
  cx = x; cy = y;
}

void f_lineto()
{
  int x, y;
  y = pop(); EC;
  x = pop(); EC;
  if (! gflag) return;
  XDrawLine(d, w0, gc[color], cx, cy, x, y);
  XFlush(d);
  cx = x; cy = y;
}

void f_rmoveto()
{
  int x, y;
  y = pop(); EC;
  x = pop(); EC;
  cx += x; cy += y;
}

void f_rlineto()
{
  int x, y;
  y = pop(); EC;
  x = pop(); EC;
  if (! gflag) return;
  XDrawLine(d, w0, gc[color], cx, cy, cx+x, cy+y);
  XFlush(d);
  cx += x; cy += y;
}

void f_rect()
{
  int x, y, w, h;
  h = pop(); EC;
  w = pop(); EC;
  y = pop(); EC;
  x = pop(); EC;
  if (! gflag) return;
  XDrawRectangle(d, w0, gc[color], x, y, w, h);
  XFlush(d);
}

void f_rectf()
{
  int x, y, w, h;
  h = pop(); EC;
  w = pop(); EC;
  y = pop(); EC;
  x = pop(); EC;
  if (! gflag) return;
  XFillRectangle(d, w0, gc[color], x, y, w, h);
  XFlush(d);
}

void f_point()
{
  int x, y;
  y = pop(); EC;
  x = pop(); EC;
  if (! gflag) return;
  XDrawPoint(d, w0, gc[color], x, y);
  XFlush(d);
}

void f_ellips()
{
  int x, y, w, h;
  h = pop(); EC;
  w = pop(); EC;
  y = pop(); EC;
  x = pop(); EC;
  if (! gflag) return;
  XDrawArc(d, w0, gc[color], x, y, w, h, 0, 360*64);
  XFlush(d);
}

void f_ellipsf()
{
  int x, y, w, h;
  h = pop(); EC;
  w = pop(); EC;
  y = pop(); EC;
  x = pop(); EC;
  if (! gflag) return;
  XFillArc(d, w0, gc[color], x, y, w, h, 0, 360*64);
  XFlush(d);
}

void f_setcolor()
{
  int c;
  c = pop(); EC;
  if (c >= 0) color = c % MAXCOL;
}

void f_setbgc()
{
  int c;
  c = pop(); EC;
  if (c >= 0) bgcolor = c % MAXCOL;
  for(c = 0; c < MAXCOL; ++c) {
    XSetBackground(d, gc[c], pxl[bgcolor]);
  }
}

void f_setfont()
{
  int c;
  c = pop(); EC;
  if (c >= 0) font = c % 2;
}

void f_drawstring()
{
  int x, y, i, len;
  static char str[LINE];

  pop_string(str); EC;
  y = pop(); EC;
  x = pop(); EC;
  len = strlen(str);
  if (! gflag) return;
  XSetFont(d, gc[color], font_dat[font].f);
  XDrawString(d, w0, gc[color],
	      x+font_dat[font].lbe, y+font_dat[font].asc, str, len);
  XFlush(d);
}

void f_drawistring()
{
  int x, y, i, len;
  static char str[LINE];

  pop_string(str); EC;
  y = pop(); EC;
  x = pop(); EC;
  len = strlen(str);
  if (! gflag) return;
  XSetFont(d, gc[color], font_dat[font].f);
  XDrawImageString(d, w0, gc[color],
		   x+font_dat[font].lbe, y+font_dat[font].asc, str, len);
  XFlush(d);
}
